#%%
#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Xu 2017
https://link.springer.com/article/10.1007/s10236-017-1122-8
showed that just a simple linear interpolation of amplitude and phase of tidal constituents is incorrect 
A simple approach by solving the two equations of interpolation based on the fact 
that the interpolation from two nearby points is linear only in time domain but in spectral domain it is non-linear.
Here we develop python program for his matlab functions.
"""
#%%
import numpy as np
# x=np.array([0,4,24,6,7,5,1,3])
# xi=np.array([9,8])
def interp1_w(x, xi):
    # interp1_w: Outputs 1-D interpolation weights and associated info. 
    
    # % Inputs: x: the data points for the interpolation
    # % xi: the points for the interpolation
    # % Output: W: a structure with fields for interpolation weights for 
    # % alpha and beta and the indices i1 and i2 to locate the 
    # % interpolation data.
    # based on Zhigang Xu, 2017/Jan/03
    n=np.argsort(x)
    x=x[n] #sorting x in ascending order and re-aarange amp and phi.

    if min(xi)<x[0] or max(xi)>x[-1]:
        print('Error:There are interpolation points outside the range of x!')

    dx=np.diff(x)
    if np.all(dx)==0:
        print('Error:x must be distinct!')

    i=np.linspace(0,len(x)-1,len(x))
    jmax=len(xi)
    W={}
    W['alpha']=np.nan*(np.ones(np.size(xi)))
    W['beta']=np.nan*(np.ones(np.size(xi)))
    W['i1']=np.zeros(np.size(xi))
    W['i2']=np.zeros(np.size(xi))

    for j in range(jmax):
        x0=xi[j]
        i1=int(max(i[x0>=x]));i2=int(min(i[x0<=x]))
        x1=x[i1];x2=x[i2];dx=x2-x1
        if dx==0:
            W['alpha'][j]=1
        else:
            W['alpha'][j]=(x2-x0)/dx
        
        W['beta'][j]=1-W['alpha'][j]
        W['i1'][j]=n[i1]
        W['i2'][j]=n[i2]
    return W

#function to interpolte 1D tidal harmonic constituents
def THCinterp1(x,amp, phi, xi,isdeg,*args):
    # THCinterp1: Interpolates 1-D tidal harmonic constituents(THC).
    # Inputs
    # x: the 1-D spatial coordinate where amp and phi are defined
    # amp and phi: amplitudes and phase lags defined at x
    # xi: the coordinates for the interpolation
    # isdeg: a logical variable to indicate whether the phase lags are in degree or not
    #  its default value is true 
    # Outputs:
    # amp and phi: interpolated amplitudes and phase lags at xi
    # Based on Zhigang Xu, 2017/Jan/03.
    
    #check for number of input arguments in function    
    varargin= args
    nargin = 5 + len(varargin)
    if (nargin<5 or nargin>6):
        print('Error:Not enough input arguments!')
        return None
    else:
        if isdeg==True:
            phi=phi*np.pi/180.
        W=interp1_w(x,xi)
        i1=W['i1'].astype(int);i2=W['i2'].astype(int)
        alpha=W['alpha'];beta=W['beta']
        acpd=amp*np.cos(phi)
        aspd=amp*np.sin(phi)
        acp=alpha*acpd[i1]+beta*acpd[i2]
        asp=alpha*aspd[i1]+beta*aspd[i2]
        phi=np.arctan2(asp,acp)
        amp=asp/np.sin(phi)
        if isdeg==True:
            phi=phi*180/np.pi
            id=phi<0
            phi[id]=phi[id]+360 #make sure the phase lags are in the range of [0,360]
    return amp, phi

# write  quick test for the function above  


#%%
# THC interpolation 2-D tidal harmonic constituents.
# the data coordinates are defined in a 2-D grid
# the interpolations points don't have to be grid points
def THCinterp2(xx,yy,amp,phi,xxi,yyi,isdeg,**args):
    # THCinterp2: Interpolates 2-D tidal harmonic constituents(THC).
    #  Inputs:
    #  xx and yy: the 2-D spatial coordinates where amp and phi are defined
    #  amp and phi: amplitudes and phase lags defined at xx and yy
    #  xxi and yyi: the coordinates for the interpolation they are both vectors
    #  isdeg: a logical variable to indicate whether the phase lags are in degree or not
    #  outputs:
    #  amp and phi: interpolated amplitudes and phase lags at xxi and yyi
    #  Based on Zhigang Xu, 2017/Jan/03.
    
    #check for number of input arguments in function    
    varargin= args
    nargin = 5 + len(varargin)
    if (nargin<5 or nargin>6):
        print('Error:Not enough input arguments!')
        return None
    else:
        if isdeg==True:
            phi=phi*np.pi/180.
        x=xx[0,:].flatten()
        y=yy[:,0].flatten()
        # Interpolation in x-direction
        acp=amp*np.cos(phi);asp=amp*np.sin(phi)
        xi=xxi
        W=interp1_w(x,xi); j1=W['i1'].astype(int); j2=W['i2'].astype(int)
        [m,n]=np.shape(asp)
        alpha=np.ones((m,1))*W['alpha']
        beta=np.ones((m,1))*W['beta']
        acp=alpha*acp[:,j1]+beta*acp[:,j2]
        asp=alpha*asp[:,j1]+beta*asp[:,j2]
        # Interpolation in y-direction
        amp=np.nan*(np.ones(np.size(xxi)))
        phi=np.nan*(np.ones(np.size(xxi)))
        for c in range(len(xi)):
            yi=[yyi[c]]
            W=interp1_w(y,yi); i1=W['i1'].astype(int); i2=W['i2'].astype(int)
            alpha=W['alpha']; beta=W['beta']
            ac=alpha*acp[i1,c]+beta*acp[i2,c]
            aspp=alpha*asp[i1,c]+beta*asp[i2,c]
            p=np.arctan2(aspp,ac)
            phi[c]=p
            amp[c]=aspp/np.sin(p)
        if isdeg==True:
            phi=phi*180/np.pi;id=phi<0
            phi[id]=phi[id]+360 #make sure the phase lags are in the range of [0,360
        return (amp, phi)
      
#%%
if __name__=='__main__':
    # Testing the 1D THC interpolation function
    T=12.
    omega=2*np.pi/T
    a1=1.5;phi_1=45;x1=0
    a2=2.0;phi_2=-45;x2=1
    xi=np.array([0.65])
    amp=np.array([a1,a2])
    phi=np.array([phi_1,phi_2])
    x=np.array([x1,x2])
    amp, phi = THCinterp1(x,amp, phi, xi,True)
    #true amp and phi
    amp_true=1.4020; phi_true=336.991 
    print('1D THC interpolation test results:')
    print('amp_true=',amp_true,'amp=',amp)
    print('phi_true=',phi_true,'phi=',phi)
    
    # Testing the 2D THC interpolation function
    x=np.array([0,1,2])
    y=np.array([0,1,2])
    xx,yy=np.meshgrid(x,y)
    amp=np.array([[0.4655,0.8315,1.1180],[1.0474,1.4135,1.700],[1.1075,1.4736,1.7601]])
    phi=np.array([[348.97,324.66,276.99],[259.50,235.1935,187.5196],[122.1608,97.8522,50.1783]])
    xi=np.array([0.5,1.5])
    yi=np.array([0.5,1.5])
    amp,phi=THCinterp2(xx,yy,amp,phi,xi,yi,True)
    # true amp and phi
    amp_true=np.array([0.6910,0.5300])
    phi_true=np.array([272.85, 137.6])
    print('2D THC interpolation test results:')
    print('amp_true=',amp_true,'amp=',amp)
    print('phi_true=',phi_true,'phi=',phi)
